@@ -3,7 +3,7 @@ BEGIN {
   $MooseX::Role::Parameterized::Meta::Trait::Parameterizable::AUTHORITY = 'cpan:SARTAK';
 }
 # ABSTRACT: trait for parameterizable roles
-$MooseX::Role::Parameterized::Meta::Trait::Parameterizable::VERSION = '1.04';
+$MooseX::Role::Parameterized::Meta::Trait::Parameterizable::VERSION = '1.05';
 use Moose::Role;
 use MooseX::Role::Parameterized::Meta::Role::Parameterized;
 use MooseX::Role::Parameterized::Parameters;
@@ -32,6 +32,7 @@ has parameters_metaclass => (
         add_parameter        => 'add_attribute',
         construct_parameters => 'new_object',
     },
+    predicate => '_has_parameters_metaclass',
 );
 
 has role_generator => (
@@ -118,6 +119,32 @@ around apply => sub {
     $role->apply($consumer, %args);
 };
 
+around reinitialize => sub {
+    my $orig  = shift;
+    my $class = shift;
+    my ($pkg) = @_;
+    my $meta  = blessed($pkg) ? $pkg : find_meta($pkg);
+
+    my $meta_meta = $meta->meta;
+
+    my %p;
+    if ( $meta_meta->can('does_role') && $meta_meta->does_role(__PACKAGE__) ) {
+        %p = map { $_ => $meta->$_ }
+            qw( parameterized_role_metaclass parameters_class );
+        $p{parameters_metaclass} = $meta->parameters_metaclass
+            if $meta->_has_parameters_metaclass;
+        $p{role_generator} = $meta->role_generator
+            if $meta->has_role_generator;
+    }
+
+    my $new = $class->$orig(
+        @_,
+        %p,
+    );
+
+    return $new;
+};
+
 1;
 
 __END__
@@ -132,7 +159,7 @@ MooseX::Role::Parameterized::Meta::Trait::Parameterizable - trait for parameteri
 
 =head1 VERSION
 
-version 1.04
+version 1.05
 
 =head1 DESCRIPTION
 
